"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _path = _interopRequireDefault(require("path"));

var _Cache = _interopRequireDefault(require("./Cache"));

var _defaults = require("./defaults");

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
const DB_VERSION = '005';
const DB_MAX_FILES = 256;
const CACHE_SIZES = {
  idx: 3 * 256,
  key: 2 * 256,
  val: 1 * 256
};

class Files {
  constructor(base, file, options) {
    this._isTrie = false;
    this._isOpen = false;
    this._fds = [];
    this._path = void 0;
    this._isTrie = options.isTrie;
    this._path = _path.default.join(base, file);

    _mkdirp.default.sync(this._path);
  }

  close() {
    this._fds.forEach(fds => Object.values(fds).forEach((_ref) => {
      let {
        fd
      } = _ref;
      return _fs.default.closeSync(fd);
    }));

    this._isOpen = false;
  }

  open() {
    for (let i = 0; i < DB_MAX_FILES; i++) {
      this._fds[i] = ['idx', 'key', 'val'].reduce((fds, type) => {
        const count = "0".concat(i.toString(16)).slice(-2);

        const file = _path.default.join(this._path, "".concat(DB_VERSION, ".").concat(count, ".").concat(type));

        if (!_fs.default.existsSync(file)) {
          _fs.default.writeFileSync(file, new Uint8Array(type === 'idx' ? _defaults.HDR_TOTAL_SIZE : 0));
        }

        const cache = new _Cache.default(CACHE_SIZES[type]);

        const fd = _fs.default.openSync(file, 'r+');

        const size = _fs.default.fstatSync(fd).size;

        fds[type] = {
          cache,
          fd,
          file,
          size
        };
        return fds;
      }, {});
    }
  }

  __append(index, type, buffer) {
    const fd = this._fds[index][type];
    const at = fd.size;

    _fs.default.writeSync(fd.fd, buffer, 0, buffer.length, at);

    fd.size += buffer.length;
    fd.cache.set(at, buffer);
    return at;
  }

  __read(index, type, at, length) {
    const fd = this._fds[index][type];
    const cached = fd.cache.get(at);

    if (cached) {
      return cached;
    }

    const buffer = new Uint8Array(length);

    _fs.default.readSync(fd.fd, buffer, 0, length, at);

    fd.cache.set(at, buffer);
    return buffer;
  }

  __update(index, type, at, buffer) {
    const fd = this._fds[index][type];

    _fs.default.writeSync(fd.fd, buffer, 0, buffer.length, at);

    fd.cache.set(at, buffer);
    return at;
  }

  _appendHdr(index, buffer) {
    return this.__append(index, 'idx', buffer) / _defaults.HDR_TOTAL_SIZE;
  }

  _appendKey(index, buffer) {
    return this.__append(index, 'key', buffer) / _defaults.KEY_TOTAL_SIZE;
  }

  _appendVal(index, buffer) {
    return this.__append(index, 'val', buffer);
  }

  _readHdr(index, at) {
    return this.__read(index, 'idx', at * _defaults.HDR_TOTAL_SIZE, _defaults.HDR_TOTAL_SIZE);
  }

  _readKey(index, at) {
    return this.__read(index, 'key', at * _defaults.KEY_TOTAL_SIZE, _defaults.KEY_TOTAL_SIZE);
  }

  _readVal(index, at, length) {
    return this.__read(index, 'val', at, length);
  }

  _updateHdr(index, at, buffer) {
    this.__update(index, 'idx', at * _defaults.HDR_TOTAL_SIZE, buffer);
  }

  _updateKey(index, at, buffer) {
    this.__update(index, 'key', at * _defaults.KEY_TOTAL_SIZE, buffer);
  }

}

exports.default = Files;